import * as Iceform from '@modal-sh/iceform-next';
import * as React from 'react';
import { useRouter } from 'next/router';
import { TextInput } from '@/components/TextInput';
import { MultilineTextInput } from '@/components/MultilineTextInput';
import { ActionButton } from '@/components/ActionButton';

export interface NotesItemPageProps {
	note: {
		id: string;
		title: string;
		content: string;
		image: string;
	},
	noscript?: boolean,
}

const NotesItemPage: Iceform.NextPage<NotesItemPageProps> = ({
	req,
	res,
	note,
	noscript,
}) => {
	const router = useRouter();
	const body = (res.body ?? note ?? {}) as Record<string, unknown>;
	const {
		response,
		loading,
		refresh: defaultRefresh,
		...isoformProps
	} = Iceform.useResponse({
		res
	});

	const [responseData, setResponseData] = React.useState<unknown>();
	React.useEffect(() => {
		// response.bodyUsed might be undefined, so we use a strict comparison
		if (
			response?.bodyUsed === false
			&& response.status !== 204
			&& response.headers.get('Content-Type')?.startsWith('application/json')
		) {
			response?.json().then((responseData) => {
				setResponseData(responseData);
			});
		}
	}, [response]);

	return (
		<div className="my-24">
			<div className="px-8 max-w-screen-sm mx-auto">
				<Iceform.Form
					{...isoformProps}
					className="contents"
					method="post"
					action={`/a/notes/${req.query.noteId}`}
					clientAction={`/api/notes/${req.query.noteId}`}
					clientMethod="put"
					refresh={defaultRefresh}
					aria-label="Edit Existing Note"
					disableFetch={noscript}
				>
					<fieldset
						className="contents"
					>
						<legend
							className="sr-only"
						>
							Edit Note
						</legend>
						<div className="flex flex-col gap-4">
							<div>
								<TextInput
									type="text"
									name="title"
									label="Title"
									className="block"
									defaultValue={body.title as string}
								/>
							</div>
							<div>
								{
									typeof body.image === 'string'
									&& body.image.length > 0
									&& (
										<img
											src={body.image}
											alt={body.title as string}
										/>
									)
								}
								<label>
									<span className="after:block">Image</span>
									<input type="file" name="image" />
								</label>
							</div>
							<div>
								<MultilineTextInput
									name="content"
									label="Content"
									className="block"
									defaultValue={body.content as string}
								/>
							</div>
							<div className="flex justify-end gap-4">
								<div>
									<ActionButton type="submit" form="delete-note-form">Delete</ActionButton>
								</div>
								<div>
									<ActionButton type="submit" className="bg-white text-black">Update</ActionButton>
								</div>
							</div>
						</div>
					</fieldset>
				</Iceform.Form>
				<Iceform.Form
					{...isoformProps}
					method="post"
					action={`/a/notes/${req.query.noteId}`}
					clientAction={`/api/notes/${req.query.noteId}`}
					clientMethod="delete"
					className="contents"
					refresh={async (response) => {
						defaultRefresh(response);
						await router.push('/notes');
					}}
					id="delete-note-form"
					aria-label="Delete Note"
					disableFetch={noscript}
				/>
			</div>
		</div>
	);
};

// TODO type safety
export const getServerSideProps = Iceform.destination.getServerSideProps({
	fn: async (actionReq, actionRes, ctx) => {
		const {noteId, noscript} = ctx.query;
		let origin: string;
		if (ctx.req.headers.referer) {
			const refererUrl = new URL(ctx.req.headers.referer as string);
			origin = refererUrl.origin;
		} else {
			// TODO how to get the scheme?
			const scheme = 'http';
			origin = `${scheme}://${ctx.req.headers.host}`;
		}
		const url = new URL(`/api/notes/${noteId}`, origin);
		const noteResponse = await fetch(url.toString(), {
			headers: {
				'Accept': 'application/json',
			}
		});
		if (noteResponse.ok) {
			const note = await noteResponse.json();
			return {
				props: {
					note,
					noscript: noscript === 'true',
				},
			};
		}

		return {
			notFound: true,
		};
	},
});

export default NotesItemPage;
