import { GetServerSideProps, NextPage } from 'next';
import * as Iceform from '@modal-sh/iceform-next';
import { useRouter } from 'next/router';
import { TextInput } from '@/components/TextInput';
import { MultilineTextInput } from '@/components/MultilineTextInput';
import { ActionButton } from '@/components/ActionButton';
import * as React from 'react';

export interface NotesPageProps {
	notes: {
		id: string;
		title: string;
		content: string;
		image: string;
	}[];
	noscript?: boolean;
}

const NotesPage: NextPage<NotesPageProps> = ({
	notes,
	noscript,
}) => {
	const router = useRouter();

	return (
		<div className="my-24">
			<div className="px-8 max-w-screen-sm mx-auto">
				<Iceform.Form
					method="post"
					action={noscript ? '/a/notes?noscript=true' : '/a/notes'}
					clientAction="/api/notes"
					refresh={async (response) => {
						if (response.status !== 201) {
							return;
						}
						const { id } = await response.json();
						await router.push(`/notes/${id}`);
					}}
					aria-label="Create New Note"
					className="contents"
					disableFetch={noscript}
				>
					<fieldset
						className="contents"
					>
						<legend
							className="sr-only"
						>
							New Note
						</legend>
						<div className="flex flex-col gap-4">
							<div>
								<TextInput
									type="text"
									name="title"
									label="Title"
									className="block"
								/>
							</div>
							<div>
								<label>
									<span className="after:block">Image</span>
									<input type="file" name="image" />
								</label>
							</div>
							<div>
								<MultilineTextInput
									name="content"
									label="Content"
									className="block"
								/>
							</div>
							<div className="text-right">
								<ActionButton type="submit" className="bg-white text-black">Submit</ActionButton>
							</div>
						</div>
					</fieldset>
				</Iceform.Form>
				<div className="mt-24 flex flex-col gap-4">
					{notes.map((note) => (
						<div
							key={note.id}
							className="border rounded p-4"
						>
							<div className="flex gap-4 justify-end">
								<form
									method="get"
									action={`/notes/${note.id}`}
									className="contents"
								>
									<div>
										<ActionButton type="submit">
											Edit
										</ActionButton>
									</div>
								</form>
								<Iceform.Form
									method="post"
									action={`/a/notes/${note.id}`}
									clientAction={`/api/notes/${note.id}`}
									clientMethod="delete"
									className="contents"
									refresh={async () => {
										await router.push('/notes');
									}}
									disableFetch={noscript}
								>
									<div>
										<ActionButton type="submit">
											Delete
										</ActionButton>
									</div>
								</Iceform.Form>
							</div>
							<div className="grid grid-cols-3 gap-4">
								<div>
									<img
										className="w-full"
										src={note.image}
										alt={note.title}
									/>
								</div>
								<div className="col-span-2">
									<div
										className="font-bold"
									>
										{note.title}
									</div>
									<div>
										{note.content}
									</div>
								</div>
							</div>
						</div>
					))}
				</div>
			</div>
		</div>
	)
};

export const getServerSideProps: GetServerSideProps = async (ctx) => {
	let origin: string;
	if (ctx.req.headers.referer) {
		const refererUrl = new URL(ctx.req.headers.referer as string);
		origin = refererUrl.origin;
	} else {
		// TODO how to get the scheme?
		const scheme = 'http';
		origin = `${scheme}://${ctx.req.headers.host}`;
	}
	const url = new URL(`/api/notes`, origin);
	const noteResponse = await fetch(url.toString(), {
		headers: {
			'Accept': 'application/json',
		}
	});
	if (noteResponse.ok) {
		const notes = await noteResponse.json();
		return {
			props: {
				notes,
				noscript: ctx.query?.noscript === 'true',
			},
		};
	}

	return {
		notFound: true,
	};
};

export default NotesPage;
