import { IncomingMessage } from 'http';
import { GetServerSideProps } from 'next';
import { NextApiRequest as DefaultNextApiRequest } from 'next/dist/shared/lib/utils';
import crypto from 'crypto';
import {
	DEFAULT_ENCTYPE_DESERIALIZERS,
	deserializeFormObjectBody,
	EncTypeDeserializerMap,
} from '../../utils/serialization';
import {
	METHOD_FORM_KEY,
	PREVENT_REDIRECT_FORM_KEY,
	REQUEST_ID_FORM_KEY,
} from '../../common/constants';
import { ACTION_STATUS_CODE, DEFAULT_METHOD } from '../constants';
import { getBody } from '../../utils/request';
import {
	ENCTYPE_APPLICATION_OCTET_STREAM,
	ENCTYPE_MULTIPART_FORM_DATA,
} from '../../common/enctypes';
import { ActionWrapperOptions } from './common';
import { IceformNextServerResponse } from '../response';
import { cacheResponse } from '../cache';

const getFormObjectMethodAndBody = async (
	req: IncomingMessage,
	deserializers: EncTypeDeserializerMap,
	methodFormKey = METHOD_FORM_KEY as string,
) => {
	const deserialized = await deserializeFormObjectBody({
		req,
		deserializers,
	});
	const {
		[methodFormKey]: method = req.method ?? DEFAULT_METHOD,
		...body
	} = deserialized as {
		[key: string]: unknown,
	};

	return {
		body,
		method,
	};
};

const getBinaryMethodAndBody = async (
	req: IncomingMessage,
) => {
	const body = await getBody(req);

	return {
		body,
		method: req.method ?? DEFAULT_METHOD,
	};
};

const isContentTypeFormObject = (
	contentType: string,
	deserializers: EncTypeDeserializerMap,
) => (
	contentType?.startsWith(`${ENCTYPE_MULTIPART_FORM_DATA};`)
	|| Object.keys(deserializers).includes(contentType)
);

/**
 * Wraps the `getServerSideProps` function to support no-script.
 * @param options
 */
export const getServerSideProps = (options: ActionWrapperOptions): GetServerSideProps => async (
	ctx,
) => {
	const {
		referer = '/',
		'content-type': contentType = ENCTYPE_APPLICATION_OCTET_STREAM,
	} = ctx.req.headers;
	const { deserializers = DEFAULT_ENCTYPE_DESERIALIZERS } = options;
	const methodAndBodyFn = (
		isContentTypeFormObject(contentType, deserializers)
			? getFormObjectMethodAndBody
			: getBinaryMethodAndBody
	);
	const { body, method } = await methodAndBodyFn(
		ctx.req,
		deserializers,
		options.methodFormKey ?? METHOD_FORM_KEY,
	);
	const req = {
		...ctx.req,
		body,
		query: {
			...ctx.query,
			...(ctx.params ?? {}),
		},
		// ?: how to prevent malicious method spoofing?
		method,
	} as DefaultNextApiRequest;
	const res = new IceformNextServerResponse(ctx.req);
	await options.fn(req, res);
	const requestId = crypto.randomBytes(8).toString('base64url');
	await cacheResponse(requestId, res);

	if (typeof options.onAction === 'function') {
		const onActionResult = await options.onAction(
			req,
			res,
			ctx,
		);
		if (onActionResult) {
			return onActionResult;
		}
	}

	const preventRedirect = (
		typeof req.body === 'object'
		&& req.body !== null
		&& (options.preventRedirectFormKey ?? PREVENT_REDIRECT_FORM_KEY) in req.body
	);
	const redirectDestinationRaw = (
		res.location
		&& typeof options.mapLocationToRedirectDestination === 'function'
		&& !preventRedirect
	)
		? options.mapLocationToRedirectDestination(referer, res.location)
		: referer;

	const url = new URL(redirectDestinationRaw, 'http://example.com');
	const redirectSearchParams = Object.fromEntries(new URLSearchParams(url.search).entries());
	const searchParams = new URLSearchParams({
		...((ctx.query ?? {}) as Record<string, string>),
		...redirectSearchParams,
	});
	searchParams.set(options.requestIdFormKey ?? REQUEST_ID_FORM_KEY, requestId);
	url.search = searchParams.toString();

	return {
		redirect: {
			destination: `/${url.pathname}${url.search}`,
			statusCode: ACTION_STATUS_CODE,
		},
		props: {
			query: ctx.query,
			body: res.data,
		},
	};
};
