import NodeCache from 'node-cache';
import { STATUS_CODES } from 'http';
import { DEFAULT_RESPONSE_STATUS_CODE } from './constants';

interface CacheableResponse {
	statusCode: number;
	statusMessage?: string;
	contentType?: string;
	data?: unknown;
}

const cache = new NodeCache({
	deleteOnExpire: true,
	stdTTL: 60,
});

export const cacheResponse = async (requestId: string, res: CacheableResponse) => {
	let cacheValue = '';
	cacheValue += `${res.statusCode.toString()} ${res.statusMessage || ''}\n`;
	if (res.contentType) {
		cacheValue += `Content-Type: ${res.contentType}\n`;
	}
	if (res.data) {
		cacheValue += `\n${res.data as string}`;
	}

	return new Promise<void>((resolve) => {
		cache.set(requestId, cacheValue);
		resolve();
	});
};

const parseCacheValue = (cacheValue: string) => {
	const [statusLine, ...headersAndBody] = cacheValue.split('\n');
	const [statusCode, ...statusMessageWords] = statusLine.split(' ');
	const statusMessage = statusMessageWords.join(' ');
	const bodyStart = headersAndBody.findIndex((line) => line === '');
	const headers = headersAndBody.slice(0, bodyStart);
	const data = headersAndBody.slice(bodyStart + 1).join('\n');
	const contentTypeHeader = headers.find((header) => header.toLowerCase().startsWith('content-type:'));
	const contentType = contentTypeHeader?.split(':')[1].trim();
	return {
		statusCode: parseInt(statusCode, 10),
		statusMessage: statusMessage || STATUS_CODES[statusCode],
		contentType,
		data,
	};
};

export const retrieveCache = async (requestId: string) => new Promise<
	CacheableResponse
>((resolve) => {
	const requestBuffer = cache.get<string>(requestId);

	if (!requestBuffer) {
		resolve({
			statusCode: DEFAULT_RESPONSE_STATUS_CODE,
			statusMessage: STATUS_CODES[DEFAULT_RESPONSE_STATUS_CODE],
			contentType: undefined,
			data: '',
		});
		return;
	}

	cache.del(requestId);
	const value = parseCacheValue(requestBuffer);
	resolve(value);
});
